import inspect
import operator
import signal
import time


class TimeoutFunctionException(Exception):
    """Exception to raise on a timeout"""
    pass

class TimeoutError(Exception):
    def __init__(self, message, elapsed=None, *args, **kwargs):
        self.elapsed = elapsed
        super(TimeoutError, self).__init__(message, *args, **kwargs)


class TimerSeconds:
    def __init__(self, time=1, timeout_message=None):
        self.time = time or 1
        self.message = timeout_message
        self.start_time = None
        self.elapsed = None
    def handle_timeout(self, signum, frame):
        if self.start_time is not None:
            self.elapsed = time.time() - self.start_time
        message  = self.message or 'TimeOut fired after "{0}/{1}" seconds!'\
            .format(self.elapsed, self.time)
        raise TimeoutError(message, elapsed=self.elapsed)
    def __enter__(self):
        self.start_time = time.time()
        signal.signal(signal.SIGALRM, self.handle_timeout)
        signal.alarm(self.time)
    def __exit__(self, type, value, traceback):
        if self.start_time is not None:
            self.elapsed = time.time() - self.start_time
        signal.alarm(0)


class WaitForResultException(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


def wait_for_result(callback,
                    result,
                    timeout=60,
                    poll_wait=10,
                    oper=operator.eq,
                    allowed_exception_types=None,
                    debug_method=None,
                    **callback_kwargs):
    """
        Repeatedly run and wait for the provided callback to return the expected result,
        or timeout

        :param callback: A function/method to run and monitor the result of
        :param result: result from the call back provided that we are looking for
        :param poll_wait:Time to wait between callback executions
        :param timeout: Time in seconds to wait before timing out and returning failure
        :param allowed_exception_types: list of exception classes that can be caught and allow
                                        the wait_for_result operation to continue
        :param oper: operator obj used to evaluate 'result' against callback's
                     result. ie operator.eq, operator.ne, etc..
        :param debug_method: optional method to use when writing debug messages
        :param callback_kwargs: optional kwargs to be provided to 'callback' when its executed
        :return: result upon success
        :raise: TimeoutFunctionException when instance does not enter proper state
        """
    if not debug_method:
        try:
            # Naughty/lazy way to log results using the caller's logger interface
            caller = inspect.currentframe().f_back.f_locals['self']
            logger = getattr(caller, 'logger', None)
            if logger:
                debug_method = logger.debug
        except Exception as E:
            pass
    debug = debug_method
    if not debug:
        def debug(msg):
            print str(msg)
    allowed_exception_types = allowed_exception_types or []
    debug("Beginning poll loop for result " + str(callback.func_name) + " to go to " +
          str(result))
    start = time.time()
    elapsed = 0
    current_state = callback(**callback_kwargs)
    while (elapsed < timeout and not oper(current_state, result)):
        debug(str(callback.func_name) + ' returned: "' + str(current_state) + '" after ' +
              str(elapsed / 60) + " minutes " + str(elapsed % 60) + " seconds.")
        debug("Sleeping for " + str(poll_wait) + " seconds")
        time.sleep(poll_wait)
        try:
            current_state = callback(**callback_kwargs)
        except allowed_exception_types as AE:
            debug('Caught allowed exception:' + str(AE))
            pass
        elapsed = int(time.time() - start)
    debug(str(callback.func_name) + ' returned: "' + str(current_state) + '" after ' +
          str(elapsed / 60) + " minutes " + str(elapsed % 60) + " seconds.")
    if not oper(current_state, result):
        raise TimeoutFunctionException(str(callback.func_name) + " did not return " +
                                       str(operator.ne.__name__) +
                                       "(" + str(result) + ") true after elapsed:" + str(elapsed))
    return current_state




class EucaAdminRequired(Exception):
    """
    Used when an operation which requires eucalyptus/admin access is attempted by a user
    which does not have the ability to execute.
    """
    def __init__(self, value=None):
        self.value = value or 'Eucalyptus administrative account required to perform ' \
                              'this operation'

    def __str__(self):
        return repr(self.value)

class EucaSysAdminRequired(Exception):
    """
    Used when an operation which requires machine login user access is attempted by a user
    which does not have the ability to execute.
    """
    def __init__(self, value=None):
        self.value = value or 'System administrative access required to perform ' \
                              'this operation'

    def __str__(self):
        return repr(self.value)



import re

f = open('convert.txt')
with f:
    text = f.readlines()
start = False
end = False
block = ""
args = []
kwargs = {}
if text:
    print "_DEFAULT_CLI_ARGS = copy.copy(CliTestRunner._DEFAULT_CLI_ARGS)"
for line in text:
    text = str(line.strip())
    text = text.replace('self.parser.add_argument(', '')
    if not text or re.match(r'^\s*$', line):
        continue
    block += text
    if re.search(r'\)\s*$', line):
        block = block.strip(')').strip()
        #print 'got end of block!: ' + str(line)
        block = re.sub(',\s*[D|d]efault\s*:\s*[\w"\']+', '', block)
        block = block.replace("''", "")
        block = block.replace('""', '')
        #print 'block:{0}'.format(block)
        args = []
        kwargs = {}
        name = None

        # end of block parse the line
        parts = block.split(",")
        for part in parts:
            #print 'GOT PART START:' + str(part)
            if re.search("'$", part):
                is_string = True
            else:
                is_string = False
            part = part.strip()

            if not part or re.match(r'^\s*$', line):
                continue
            #print 'GOT PART:' + str(part)
            match = re.search('(\w.*\w)\s*=\s*([\w\'"].*\w)', part)
            if match:
                if is_string:
                    value = "{0}".format(match.groups()[1].strip())
                    #print 'is string: ' + str(value)
                else:
                    value = match.groups()[1].strip()
                if re.search("^\s*['\"]", value) and not value.endswith(value[0]):
                    value += value[0]
                kwargs[match.groups()[0].strip()] = value
                #print 'ADDED TO KWARGS:' + str(part)
            else:
                value = part
                match = re.search("^[\s'\"]*(-+)", value)
                if not name:
                    name = part.replace("-", "")
                if match:
                    value = value.lstrip("-")
                    value = str(value).replace("_", "-")
                    #value = str(match.groups()[0]) + value
                args.append(value)
                #print 'ADDED TO ARGS:' + str(part)
        #print "name:{0}\nargs:{1}\nkwargs{2}\n".format(name, args, kwargs)
        kwarg_str = ",\n                ".join("'{0}': {1}".format(key, kwargs[key]) for key in kwargs.keys())
        #print kwarg_str
        print "_DEFAULT_CLI_ARGS[{0}] = ".format(name) + "{"
        print "    'args': [{0}],".format(", ".join("{0}".format(x) for x in args))
        print "    'kwargs': { " + kwarg_str + " }}\n"
        block = ""
        #print "\n"
